{  
  classes and types for graphics used in molecule man

}
  
unit umolgraphics;

interface

uses QGraphics, uGraphic, Types;

const
  MOL_LOCATION_SIZE = 7;
  MOL_MAX_GRAPHIC_SIZE = 8;
  NUM_PLAYER_ANIMS = 4;
  
type

  // the four movement directions allowed for the player graphic
  tMovementDir = (tmdUpLeft, tmdUpRight, tmdDownRight, tmdDownLeft);

  // array of filenames for the player graphics
  tPlayerGraphFNameArray =  Array [tmdUpLeft .. tmdDownLeft, 1 .. NUM_PLAYER_ANIMS]  of String;

  // an array for holding all the game graphics, some of which will be animated
  tGameGraphicsArray = Array of Array of TBitmap;

  // array to hold all the player bitmaps
  tMolManGraphicsArray = Array [tmdUpLeft .. tmdDownLeft, 1 .. NUM_PLAYER_ANIMS]  of TBitmap;

  // basic graphic object 
  tMolBaseStaticSprite = class (TGraphicObject)
    private
      fDestructible : Boolean;  // specifies whether object can be blown up

    public
      property   Destructible : Boolean Read fDestructible Write fDestructible;
      function   GetScreenCoords : tPoint; override; 
  End;

  // graphic object that doesn't animte
  tMolStaticSprite = class (tMolBaseStaticSprite)
    private
      // flags specifying the objects properties
      fPillMachine  : Boolean;  
      fBombMachine  : Boolean;
      fCoin         : Boolean;
      fTeleporter   : Boolean;
      
    public
      property IsPillMachine  : Boolean Read fPillMachine Write fPillMachine;
      property IsBombMachine  : Boolean Read fBombMachine Write fBombMachine;
      property IsCoin         : Boolean Read fCoin Write fCoin;
      property IsTeleporter   : Boolean Read fTeleporter Write fTeleporter;
  end;
  
  // graphic object that is animated 
  tMolAnimatedSprite = class (tMolBaseStaticSprite)
    private
      fNumFrames    : Integer;  // number of frames in the animation
      fCurrentFrame : Integer;  // current frame number
      fAnimDelay    : Integer;  // delay between frame changes
      fCurrentDelay : Integer;  // current delay counter
      fGraphics     : Array of TBitmap;  // graphics
      fCircuit      : Boolean; 

    public
      
      procedure  draw; override;
      procedure  SetAnimInfo (NumFrames, AnimDelay : Integer);
      function   SetGraphics (Filenames : Array of String) : Boolean; overload;
      function   SetGraphics (GraphicsArray : Array of tBitmap) : Boolean; overload;
      procedure  animate;
      destructor Destroy; override;

      property IsCircuit      : Boolean Read fCircuit Write fCircuit;
      
    end;

  // the Molecule Man graphic object                          
  tMolPlayerSprite = class (TMolBaseStaticSprite)
    private
      fGraphics : tMolManGraphicsArray;
      fFacing   : tMovementDir;               
      fMoving   : Boolean;      // set when the object is moving
      fAnimFrame: Integer;
      fAnimDelay: Integer;

    public
      procedure draw; override;
      destructor Destroy; override;
      function   SetGraphics (Filenames :  tPlayerGraphFNameArray) : boolean; overload;
      function   SetGraphics (GraphicsArray : tMolManGraphicsArray) : Boolean; overload;
      procedure  Move;
      procedure  StartMoving;
      procedure  StopMoving;
      property   Moving : Boolean Read fMoving;
      property   Facing : tMovementDir Read fFacing Write fFacing;
      property   AnimDelay : Integer Read fAnimDelay Write FAnimDelay;
      property   AnimFrame : Integer Read fAnimFrame Write fAnimFrame;
      
    End;
    
  // object used to hold the graphic objects in the current location    
  tMolGraphicArray = class (TObject)
    private
      // array which holds the details of the objects in the current location.
      // objects on screen occupy the same location in the array
      fArray : array [0 .. MOL_LOCATION_SIZE -1, 0 .. MOL_LOCATION_SIZE -1] of tGraphicObject;

      procedure ClearArray;

    public
      constructor Create;
      
      procedure   Place_Graphic (GraphicObject : TGraphicObject);  
      procedure   Remove_Graphic (GraphicObject : TGraphicObject);
      procedure   SetGraphicsCanvas (Canvas : TCanvas);
      procedure   AnimateGraphics;
      procedure   DrawGraphics (DrawDir : TMovementDir);
      procedure   FreeGRaphics;
      function    GetGraphic (LocXPos, LocYPos : Integer) : TGraphicObject;
  End;
  
implementation

(************************************************************************)

function tMolBaseStaticSprite.GetScreenCoords : tPoint;
Begin
  // work out where to print the object on screen
  Result.X := 224 - (((MOL_LOCATION_SIZE-1)*8) - XWorld)*4  + (YWorld) *4;
  Result.Y := (((MOL_LOCATION_SIZE-1)*8) -xWorld) * 2 + (YWorld)*2 + (8 - hghtWorld)*2+ 20;
End;

(************************************************************************)

procedure tMolAnimatedSprite.animate;
begin
  // do we need to move to the next frame ?
  inc (fCurrentDelay);
  if fCurrentDelay = fAnimDelay then
  Begin
    fCurrentDelay := 0;
    inc (fCurrentFrame);
    if fCurrentFRame = fNumFRames Then
      fCurrentFrame := 0;
  End;
End;

// draw the current frame     
procedure tMolAnimatedSprite.draw;
var
  tmpCoord : TPoint;
  
begin
  tmpCoord := GetScreenCoords;
  Canvas.Draw(tmpCoord.X, tmpCoord.Y, fGraphics[fCurrentFrame]);
end;

destructor tMolAnimatedSprite.Destroy;
var
  iLoop : Integer;
  
begin
  // do we have to free the graphics ?
  if not ClearGraphics Then exit;
  
  // clear out the graphics
  Try
    For iLoop := 0 to Length (fGraphics)-1 do
      If Assigned (FGraphics[iLoop]) Then
        FGraphics[iLoop].Free;
  Except
    Exit;
  End;

  inherited;
end;

function tMolAnimatedSprite.SetGraphics (GraphicsArray : Array of tBitmap) : Boolean; 
var
  iLoop : Integer;
  
Begin
  Result := TRUE;

  // assign the graphics for the object from a pre-loaded array
  SetLength (fGraphics, Length (GraphicsArray));
  for iLoop := 0 to Length (GraphicsArray)-1 do
    fGraphics[iLoop] := GraphicsArray[iLoop];
  ClearGRaphics := FALSE;
End;

// load the graphics on demand
function tMolAnimatedSprite.SetGraphics(Filenames : Array Of String) : Boolean;
var
  iLoop : Integer;
  
Begin
  Result :=FALSE;

  ClearGRaphics := TRUE;
  
  // initialise the fGraphics array
  SetLength (fGraphics, Length (Filenames));
  For iLoop := 0 to Length (fGRaphics) -1 do
    fGraphics [iLoop] := NIL;
    
  // now load in the graphics specified in the filenames array
  try
    For iLoop := 0 to Length (FGraphics)-1 do
    Begin
      fGraphics[iLoop] := TBitmap.Create;
      fGraphics[iLoop].LoadFromFile(Filenames[iLoop]) 
    End;
  except
    Exit;
  End;

  Result := TRUE;
  
end;

procedure tMolAnimatedSprite.SetAnimInfo(NumFRames, AnimDelay : Integer);
Begin
  fNumFrames :=NumFrames;
  fCurrentFrame := 0;
  fAnimDelay := AnimDElay;
  fCurrentDelay := 0;
End;

(************************************************************************)

procedure TMolGraphicArray.ClearArray;
var
  iLoop  : Integer;
  iLoop1 : Integer;
Begin
  For iLoop := 0 to MOL_LOCATION_SIZE -1 do
    For iLoop1 := 0 to MOL_LOCATION_SIZE -1 do
      fArray[iLoop, iLoop1] := NIL;
End;

constructor TMolGRaphicArray.Create;
Begin
  ClearArray;
End;

//take a graphic and place in the correct position within fArray
procedure  TMolGraphicArray.Place_Graphic (GraphicObject : TGraphicObject); 
Begin
  fArray[GraphicObject.xWorld div MOL_MAX_GRAPHIC_SIZE, 
         GraphicObject.yWorld div MOL_MAX_GRAPHIC_SIZE] := GraphicObject;
End;

procedure  tMolGraphicArray.Remove_Graphic (GraphicObject : TGraphicObject);
Begin
  fArray[GraphicObject.xWorld div MOL_MAX_GRAPHIC_SIZE, 
         GraphicObject.yWorld div MOL_MAX_GRAPHIC_SIZE] := NIL;
End;

function  tMolGraphicArray.GetGraphic (LocXPos, LocYPos : Integer) : TGraphicObject;
Begin
  Result := fArray [LocXpos, LocYPos];
End;

// run through the graphics, animating any animated ones
procedure tMolGraphicArray.AnimateGraphics;
var
  iLoop, iLoop1 : Integer;

Begin
  for iLoop:= 0 To MOL_LOCATION_SIZE-1 do
    for iLoop1 := 0 To MOL_LOCATION_SIZE-1 do
      if Assigned (fArray[iLoop, iLoop1]) Then
        if (fArray[iLoop, iLoop1]) is TMolAnimatedSprite Then
          With fArray[iLoop, iLoop1] As TMolAnimatedSprite do
            Animate;
End;
  
// draws the graphics in the correct order to maintain the 3d perspective
procedure   tMolGraphicArray.DrawGraphics (DrawDir : TMovementDir);
  procedure DrawifAssigned (x, y : Integer);
  Begin
    if Assigned (FArray[x,y]) Then
      fArray[x,y].Draw;
  End;
  
var
  iLoop  : Integer;
  iLoop1 : Integer;
  
Begin
  // the screen is drawn in the direction the player graphic is facing/moving
  // so that the player graphic remains in the correct relation (behind or
  // in front) to other graphics

  case DrawDir of
    tmdUpLeft, tmdDownRight :
      For iLoop := MOL_LOCATION_SIZE-1 downto 0 do
        for iLoop1 := 0 To Mol_LOCATION_SIZE-1 do
          DrawIfAssigned (iLoop, iLoop1);

    tmdUpRight, tmdDownLeft :
      For iLoop := 0 to MOL_LOCATION_SIZE-1 do
        for iLoop1 := MOL_LOCATION_SIZE-1 downto 0 do
          DrawIfAssigned (iLoop1, iLoop);
        
  End;           
End;

procedure   tMolGraphicArray.SetGraphicsCanvas (Canvas : TCanvas);
var
  iLoop : Integer;
  iLoop1 : Integer;
  
begin
  For iLoop := 0 to MOL_LOCATION_SIZE -1 do
    for iLoop1 := 0 to MOL_LOCATION_SIZE -1 do
      if Assigned (fArray[iLoop, iLoop1]) Then
        fArray [iLoop, iLoop1].SetCanvas (Canvas);
End;

procedure tMolGraphicArray.FreeGraphics;
var
  iLoop : Integer;
  iLoop1 : Integer;
  
begin
  For iLoop := 0 to MOL_LOCATION_SIZE -1 do
    for iLoop1 := 0 to MOL_LOCATION_SIZE -1 do
      if Assigned (fArray[iLoop, iLoop1]) Then
       fArray[iLoop, iLoop1].Free;
end;

(************************************************************************)

procedure tMolPlayerSprite.draw;
var
  tmpCoord : TPoint;
  
begin
  tmpCoord := GetScreenCoords;
  Canvas.Draw(tmpCoord.X, tmpCoord.Y, fGraphics[fFacing, fAnimFrame]);
End;

destructor tMolPlayerSprite.Destroy;
var
  iLoop : Integer;
  imd   : TMovementDir;
  
Begin

  if not ClearGraphics Then exit;

  // clear out the graphics
  Try
    For imd := tmdUpLeft to tmdDownLeft do
      For iLoop := 1 to NUM_PLAYER_ANIMS do
        If Assigned (FGraphics[imd, iLoop]) Then
          FGraphics[imd, iLoop].Free;
  Except
    Exit;
  End;

  inherited;
End;

function tMolPlayerSprite.SetGraphics (GraphicsArray : tMolManGraphicsArray) : Boolean; 
Begin
  Result := TRUE;
  ClearGraphics := FALSE;
  fGraphics := GraphicsArray;
End;
  
// load the player graphics for their respective files
function tMolPlayerSprite.SetGraphics (Filenames :  tPlayerGraphFNameArray) : Boolean;
var
  imd   : TMovementDir;
  iLoop : Integer;
  
Begin
  Result :=FALSE;
  ClearGraphics := TRUE;
                         
  for imd :=tMdUpLeft to tMdDownLeft do
    For iLoop := 1 to NUM_PLAYER_ANIMS do
      fGraphics [imd, iLoop] := NIL;
    
  // now load in the graphics specified in the filenames array
  try
    for imd := tmdUpLeft to tmdDownLeft do
      For iLoop := 1 to NUM_PLAYER_ANIMS do
      Begin
        fGraphics[imd, iLoop] := TBitmap.Create;
        fGraphics[imd, iLoop].LoadFromFile(Filenames[imd, iLoop]) 
      End;
  except
    Exit;
  End;

  Result := TRUE
End;

procedure  tMolPlayerSprite.StartMoving;
Begin
  fMoving := TRUE;  
  fAnimFrame := 1;
End;

procedure  tMolPLayerSprite.StopMoving;
Begin
  fMoving := FALSE; 
  fAnimFRame := 1;
End;

procedure tMolPlayerSprite.Move;
Begin

  // move the player graphic according to the direction it's facing
  // and stop when it's moved to the next position
  case FFacing of
    tMdUpLeft      : Begin
                       yWorld := yWorld -1;
                       if yWorld MOD MOL_MAX_GRAPHIC_SIZE=0 Then
                         StopMoving;
                     End;
    tMdUpRight     : Begin
                       xworld := xWorld +1;
                       if xWorld MOD MOL_MAX_GRAPHIC_SIZE=0 Then
                         StopMoving;
                       
                     End;
    tMdDownRight   : Begin 
                      yWorld := yWorld +1;
                       if yWorld MOD MOL_MAX_GRAPHIC_SIZE=0 Then
                         StopMoving;
                      
                     End;
    tMdDownLeft    : Begin
                      xWorld := xWorld -1;
                       if xWorld MOD MOL_MAX_GRAPHIC_SIZE=0 Then
                         StopMoving;
                     End;
  End;              

  // the player graphic is animated while it's moving
  if fMoving Then
  Begin
    fAnimFrame := fAnimFrame +1;
    if fAnimFrame > NUM_PLAYER_ANIMS Then
      fAnimFrame := 1;
  End;
  
end;

end.
